#pragma once

#include "plugin.h"

#define _LOG_EXIT(v)             \
    _Generic((v),                \
        int: log_exit_int,       \
        u_int64_t: log_exit_u64, \
        u_int32_t: log_exit_u32, \
        void*: log_exit_ptr,     \
        boolean: log_exit_bool,  \
        char*: log_exit_str,     \
        const char*: log_exit_str)(__FUNCTION__, v)

#define RETURN_LOGGED(v) \
    do {                 \
        _LOG_EXIT(v);    \
        return v;        \
    } while (0)

#define LOG_EXIT(v) (_LOG_EXIT(v), v)

#define false ((u_int8_t)0)
#define true ((u_int8_t)1)

static inline void log_exit_int(const char* function, int x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return value = %d\n", function, x);
}
static inline void log_exit_u64(const char* function, u_int64_t x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return value = %lu\n", function, x);
}
static inline void log_exit_u32(const char* function, u_int32_t x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return value = %u\n", function, x);
}
static inline void log_exit_ptr(const char* function, void* x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return pointer = %p\n", function, x);
}
static inline void log_exit_bool(const char* function, boolean x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return is %s\n", function, x ? "true" : "false");
}
static inline void log_exit_str(const char* function, const char* x) {
    EngFncs->write_log_entry(ENTRY_EXIT, &imsm_plugin, "%s: Exit.  Return string = %s\n", function, x);
}
