#include "sb.h"

#include <string.h>

#include "log_utils.h"

size_t sizeof_imsm_map(imsm_map_t* map) {
    return sizeof(imsm_map_t) + sizeof(u_int32_t) * (map->num_members - 1);
}

imsm_map_t* get_imsm_map(imsm_dev_t* dev, int second_map) {
    /* A device can have 2 maps if it is in the middle of a migration.
     * If second_map is:
     *    MAP_0 - we return the first map
     *    MAP_1 - we return the second map if it exists, else NULL
     *    MAP_X - we return the second map if it exists, else the first
     */
    imsm_map_t* map = &dev->vol.map[0];
    imsm_map_t* map2 = NULL;

    if (dev->vol.migr_state)
        map2 = (void*)map + sizeof_imsm_map(map);

    switch (second_map) {
    case MAP_0: break;
    case MAP_1: map = map2; break;
    case MAP_X:
        if (map2)
            map = map2;
        break;
    default: map = NULL;
    }
    return map;
}

/* return the size of the device.
 * migr_state increases the returned size if map[0] were to be duplicated
 */
size_t sizeof_imsm_dev(imsm_dev_t* dev, int migr_state) {
    size_t size = sizeof(*dev) - sizeof(imsm_map_t) + sizeof_imsm_map(get_imsm_map(dev, MAP_0));

    /* migrating means an additional map */
    if (dev->vol.migr_state)
        size += sizeof_imsm_map(get_imsm_map(dev, MAP_1));
    else if (migr_state)
        size += sizeof_imsm_map(get_imsm_map(dev, MAP_0));

    return size;
}

imsm_dev_t* get_imsm_dev(imsm_super_t* mpb, u_int8_t index) {
    int offset;
    int i;
    void* _mpb = mpb;

    if (index >= mpb->num_raid_devs)
        return NULL;

    /* devices start after all disks */
    offset = ((void*)&mpb->disk[mpb->num_disks]) - _mpb;

    for (i = 0; i <= index; i++)
        if (i == index)
            return _mpb + offset;
        else
            offset += sizeof_imsm_dev(_mpb + offset, 0);

    return NULL;
}

u_int32_t sb_get_effective_family(imsm_super_t* sb) {
    return sb->orig_family_num == 0 ? sb->family_num : sb->orig_family_num;
}

boolean sb_equal(imsm_super_t* lhs, imsm_super_t* rhs) {
    LOG_ENTRY();

    if (lhs->num_raid_devs != rhs->num_raid_devs || memcmp(lhs->sig, rhs->sig, sizeof(lhs->sig)) != 0 ||
        sb_get_effective_family(lhs) != sb_get_effective_family(rhs))
        return LOG_EXIT(false);

    return LOG_EXIT(true);
}

static u_int64_t join_u32(u_int32_t lo, u_int32_t hi) {
    return (u_int64_t)DISK_TO_CPU32(lo) | (((u_int64_t)DISK_TO_CPU32(hi)) << 32);
}

sector_count_t get_imsm_dev_size(imsm_dev_t* dev) {
    return join_u32(dev->size_low, dev->size_high);
}

sector_count_t get_imsm_map_blocks_per_member(imsm_map_t* map) {
    return join_u32(map->blocks_per_member_lo, map->blocks_per_member_hi);
}

sector_count_t get_imsm_map_pba_of_lba0(imsm_map_t* map) {
    return join_u32(map->pba_of_lba0_lo, map->pba_of_lba0_hi);
}
